#![allow(dead_code)]
use openapi_type::OpenapiType;

macro_rules! test_type {
	($ty:ty = $json:tt) => {
		paste::paste! {
			#[test]
			fn [< $ty:lower >]() {
				let schema = <$ty as OpenapiType>::schema();
				let schema = openapi_type::OpenapiSchema::into_schema(schema);
				let schema_json = serde_json::to_value(&schema).unwrap();
				let expected = serde_json::json!($json);
				assert_eq!(schema_json, expected);
			}
		}
	};
}

#[derive(OpenapiType)]
struct UnitStruct;
test_type!(UnitStruct = {
	"type": "object",
	"title": "UnitStruct",
	"additionalProperties": false
});

#[derive(OpenapiType)]
struct SimpleStruct {
	foo: String,
	bar: isize
}
test_type!(SimpleStruct = {
	"type": "object",
	"title": "SimpleStruct",
	"properties": {
		"foo": {
			"type": "string"
		},
		"bar": {
			"type": "integer"
		}
	},
	"required": ["foo", "bar"]
});

#[derive(OpenapiType)]
enum EnumWithoutFields {
	Success,
	Error
}
test_type!(EnumWithoutFields = {
	"type": "string",
	"title": "EnumWithoutFields",
	"enum": [
		"Success",
		"Error"
	]
});

#[derive(OpenapiType)]
enum EnumWithOneField {
	Success { value: isize }
}
test_type!(EnumWithOneField = {
	"type": "object",
	"title": "EnumWithOneField",
	"properties": {
		"Success": {
			"type": "object",
			"properties": {
				"value": {
					"type": "integer"
				}
			},
			"required": ["value"]
		}
	},
	"required": ["Success"]
});
